import argparse
import os
import shutil

from colorama import Fore, init

yellow = Fore.YELLOW
green = Fore.GREEN
red = Fore.RED

init(autoreset=True)

base_dir = os.getcwd()
files_in_base_dir = os.listdir(base_dir)

current_file_path = os.path.abspath(__file__)
current_file_dir = os.path.dirname(current_file_path)


def klprint(string):
    # Kivy Logger
    print(f"{green}[KIVY RELOADER]{Fore.RESET} {string}")


def create_settings_file():
    """
    Creates a copy of kivy-reloader.toml in the project folder called kivy-reloader.toml
    so the user can change the settings. If the file already exists,
    it creates a kivy-reloader-template.toml file instead.
    """

    if "kivy-reloader.toml" in files_in_base_dir:
        klprint(
            "kivy-reloader.toml already exists, creating kivy-reloader-template.toml"
        )
        shutil.copyfile(
            os.path.join(current_file_dir, "kivy-reloader.toml"),
            os.path.join(base_dir, "kivy-reloader-template.toml"),
        )
    else:
        klprint(
            "kivy-reloader.toml not found, creating it on current working directory"
        )
        shutil.copyfile(
            os.path.join(current_file_dir, "kivy-reloader.toml"),
            os.path.join(base_dir, "kivy-reloader.toml"),
        )


def create_buildozer_spec_file():
    """
    Creates a copy of buildozer.spec in the project folder if it doesn't exist
    If it exists, it creates a buildozer_template.spec file
    """
    if "buildozer.spec" in files_in_base_dir:
        klprint(
            "buildozer.spec found, creating a copy of buildozer.spec called buildozer_template.spec"
        )
        if "buildozer_template.spec" not in files_in_base_dir:
            shutil.copyfile(
                os.path.join(current_file_dir, "buildozer.spec"),
                os.path.join(base_dir, "buildozer-template.spec"),
            )
    else:
        klprint("buildozer.spec not found, creating it on current working directory")
        shutil.copyfile(
            os.path.join(current_file_dir, "buildozer.spec"),
            os.path.join(base_dir, "buildozer.spec"),
        )


def main():
    parser = argparse.ArgumentParser(description="Kivy Reloader CLI")
    subparsers = parser.add_subparsers(dest="command")
    init_parser = subparsers.add_parser("init", help="Initializes Kivy Reloader")
    args = parser.parse_args()

    if args.command == "init":
        create_settings_file()
        create_buildozer_spec_file()



import os
from typing import Any, List

import toml


class Config:
    def __init__(self):
        self.config_file = os.path.join(os.getcwd(), "kivy-reloader.toml")
        self.constants_to_import = [
            "WATCHED_FILES",
            "WATCHED_FOLDERS",
            "WATCHED_FOLDERS_RECURSIVELY",
            "WATCHED_KV_FOLDERS",
            "WATCHED_KV_FOLDERS_RECURSIVELY",
            "HOT_RELOAD_ON_PHONE",
            "STREAM_USING",
            "PORT",
            "PHONE_IPS",
            "WINDOW_TITLE",
            "SHOW_TOUCHES",
            "STAY_AWAKE",
            "TURN_SCREEN_OFF",
            "ALWAYS_ON_TOP",
            "WINDOW_X",
            "WINDOW_Y",
            "WINDOW_WIDTH",
            "SERVICE_FILES",
            "SERVICE_NAMES",
        ]
        self._load_config()

    def _load_config(self):
        if os.path.exists(self.config_file):
            with open(self.config_file, "r") as f:
                self.config = toml.load(f)["kivy_reloader"]
        else:
            raise FileNotFoundError(
                f"Config file not found: {self.config_file}. Please execute `kivy-reloader init` first."
            )

    def get(self, key: str, default: Any = None) -> Any:
        return self.config.get(key, default)

    def set(self, key: str, value: Any):
        self.config[key] = value

    def save(self):
        with open(self.config_file, "w") as f:
            toml.dump(self.config, f)

        self._load_config()

        return self

    @property
    def WATCHED_FILES(self) -> List[str]:
        return self.get("WATCHED_FILES", [])

    @property
    def WATCHED_FOLDERS(self) -> List[str]:
        return self.get("WATCHED_FOLDERS", [])

    @property
    def WATCHED_FOLDERS_RECURSIVELY(self) -> List[str]:
        return self.get("WATCHED_FOLDERS_RECURSIVELY", [])

    @property
    def WATCHED_KV_FOLDERS(self) -> List[str]:
        return self.get("WATCHED_KV_FOLDERS", [])

    @property
    def WATCHED_KV_FOLDERS_RECURSIVELY(self) -> List[str]:
        return self.get("WATCHED_KV_FOLDERS_RECURSIVELY", [])

    @property
    def HOT_RELOAD_ON_PHONE(self) -> bool:
        return self.get("HOT_RELOAD_ON_PHONE", False)

    @property
    def STREAM_USING(self) -> str:
        return self.get("STREAM_USING", "")

    @property
    def PORT(self) -> int:
        return self.get("PORT", 5555)

    @property
    def PHONE_IPS(self) -> List[str]:
        return self.get("PHONE_IPS", [])

    @property
    def WINDOW_TITLE(self) -> str:
        return self.get("WINDOW_TITLE", "Kivy Reloader")

    @property
    def SHOW_TOUCHES(self) -> bool:
        return self.get("SHOW_TOUCHES", False)

    @property
    def STAY_AWAKE(self) -> bool:
        return self.get("STAY_AWAKE", False)

    @property
    def TURN_SCREEN_OFF(self) -> bool:
        return self.get("TURN_SCREEN_OFF", False)

    @property
    def ALWAYS_ON_TOP(self) -> bool:
        return self.get("ALWAYS_ON_TOP", True)

    @property
    def WINDOW_X(self) -> str:
        return self.get("WINDOW_X", "1200")

    @property
    def WINDOW_Y(self) -> str:
        return self.get("WINDOW_Y", "100")

    @property
    def WINDOW_WIDTH(self) -> str:
        return self.get("WINDOW_WIDTH", "280")

    @property
    def SERVICE_FILES(self) -> List[str]:
        return self.get("SERVICE_FILES", [])

    @property
    def SERVICE_NAMES(self) -> List[str]:
        return self.get("SERVICE_NAMES", [])

    @property
    def FOLDERS_AND_FILES_TO_EXCLUDE_FROM_PHONE(self) -> List[str]:
        return self.get(
            "FOLDERS_AND_FILES_TO_EXCLUDE_FROM_PHONE",
            [
                "*.pyc",
                "__pycache__",
                ".buildozer",
                ".venv",
                ".vscode",
                ".git",
                ".pytest_cache",
                ".DS_Store",
                ".env",
                ".dmypy.json",
                ".mypy_cache/",
                ".dmypy.json",
                "dmypy.json",
                "env/",
                "ENV/",
                "env.bak/",
                "venv/",
                "venv.bak/",
                "bin",
                "buildozer.spec",
                "poetry.lock",
                "pyproject.toml",
                "temp",
                "tests",
                "app_copy.zip",
                "send_app_to_phone.py",
                ".gitignore",
                "README.md",
            ],
        )


config = Config()



import trio
from colorama import Fore, Style, init
from constants import PHONE_IPS

red = Fore.RED
green = Fore.GREEN
yellow = Fore.YELLOW
white = Fore.WHITE
init(autoreset=True)


async def connect_to_server(IP):
    try:
        PORT = 8050
        print(f"Connecting to IP: {green}{IP}{white} and PORT: {green}{PORT}")
        with trio.move_on_after(1):
            client_socket = await trio.open_tcp_stream(IP, PORT)
            return client_socket
    except Exception as e:
        print(f"{red}Error: {e}")
        return None


async def send_app():
    print("*" * 50)
    print(green + "Connecting to smartphone...")
    for IP in PHONE_IPS:
        client_socket = await connect_to_server(IP)
        if not client_socket:
            print(yellow + "Couldn't connect to smartphone")
            return
        print(yellow + f"Phone connected successfully: ", IP)
        print(f"\n{green}Sending app to smartphone...")
        CHUNK_SIZE = 4096
        with open(
            "app_copy.zip",
            "rb",
        ) as myzip:
            for chunk in iter(lambda: myzip.read(CHUNK_SIZE), b""):
                print("Sending chunk")
                await client_socket.send_all(chunk)
        print(green + "Finished sending app!")
    print("\n")
    print(yellow + f"Sent app to {len(PHONE_IPS)} smartphone(s)")
    print("*" * 50)


trio.run(send_app)



import os
import subprocess

import trio
from kivy.factory import Factory as F
from kivy.lang import Builder
from kivy.logger import Logger
from kivy.utils import platform

from .config import config
from .utils import get_kv_files_paths


class Reloader(F.Screen):
    pass


# fmt: off
kv = Builder.load_string("""
<Reloader>:
    screen_manager: screen_manager
    ScreenManager:
        id: screen_manager
"""
)
# fmt: on

if platform != "android":
    import inspect
    import logging
    from shutil import copytree, ignore_patterns, rmtree

    from kaki.app import App
    from kivy.base import async_runTouchApp
    from kivy.core.window import Window

    from .utils import get_auto_reloader_paths

    Window.always_on_top = True
    logging.getLogger("watchdog").setLevel(logging.ERROR)

    # from .constants import FOLDERS_AND_FILES_TO_EXCLUDE_FROM_PHONE

    # Desktop BaseApp
    class App(App):
        def __init__(self, *args, **kwargs):
            super().__init__(*args, **kwargs)

            self.DEBUG = 1
            self.AUTORELOADER_PATHS: list = get_auto_reloader_paths()
            self.HOT_RELOAD_ON_PHONE: bool = config.HOT_RELOAD_ON_PHONE
            self.KV_FILES: list = get_kv_files_paths()

            self._build()

        def _build(self):
            Logger.info("Reloader: Building the first screen")
            if self.DEBUG:
                Logger.info("Kaki: Debug mode activated")
                self.enable_autoreload()
                self.patch_builder()
                self.bind_key(286, self.rebuild)
            if self.FOREGROUND_LOCK:
                self.prepare_foreground_lock()

            self.state = {}
            self.approot = None

            if self.IDLE_DETECTION:
                self.install_idle(timeout=self.IDLE_TIMEOUT)

        async def async_run(self, async_lib="trio"):
            async with trio.open_nursery() as nursery:
                Logger.info("Reloader: Starting Async Kivy app")
                self.nursery = nursery
                self._run_prepare()
                await async_runTouchApp(async_lib=async_lib)
                self._stop()

        async def main(self) -> None:
            """
            Starts the async Kivy app
            """
            async with trio.open_nursery() as nursery:
                Logger.info("Reloader: Starting Async Kivy app")
                server = self
                server.nursery = nursery
                await server.async_run("trio")

        def rebuild(self, *args, **kwargs):
            Logger.info("Reloader: Rebuilding the application")
            first = kwargs.get("first", False)
            try:
                if not first:
                    self.unload_app_dependencies()

                Builder.rulectx = {}

                self.load_app_dependencies()
                self.set_widget(None)
                self.approot = self.build()
                self.set_widget(self.approot)
                self.apply_state(self.state)
                if self.HOT_RELOAD_ON_PHONE:
                    self.send_app_to_phone()
            except Exception as e:
                import traceback

                Logger.exception("Reloader: Error when building app")
                self.set_error(repr(e), traceback.format_exc())
                if not self.DEBUG and self.RAISE_ERROR:
                    raise

        def set_widget(self, wid):
            """
            Clear the root container, and set the new approot widget to `wid`
            """
            self.root.clear_widgets()
            self.approot = wid
            if wid is None:
                return
            self.root = self.get_root()
            self.root.add_widget(self.approot)
            Window.add_widget(self.root)
            try:
                wid.do_layout()
            except Exception:
                pass

        def clear_temp_folder_and_zip_file(self, folder, zip_file):
            if os.path.exists(folder):
                rmtree(folder)
            if os.path.exists(zip_file):
                os.remove(zip_file)

        def send_app_to_phone(self):
            # Creating a copy of the files on `temp` folder
            source = os.getcwd()
            destination = os.path.join(os.getcwd(), "temp")
            zip_file = os.path.join(os.getcwd(), "app_copy.zip")

            self.clear_temp_folder_and_zip_file(destination, zip_file)

            copytree(
                source,
                destination,
                ignore=ignore_patterns(
                    *config.FOLDERS_AND_FILES_TO_EXCLUDE_FROM_PHONE,
                ),
            )

            # Zipping all files inside `temp` folder, except the `temp` folder itself
            subprocess.run(
                f"cd {destination} && zip -r ../app_copy.zip ./* -x ./temp",
                shell=True,
                stdout=subprocess.DEVNULL,
            )

            # Sending the zip file to the phone
            path_of_current_file = inspect.currentframe().f_back
            path_of_send_app = os.path.join(
                os.path.dirname(
                    os.path.abspath(path_of_current_file.f_code.co_filename)
                ),
                "send_app_to_phone.py",
            )
            subprocess.run(f"python {path_of_send_app}", shell=True)

            # Deleting the temp folder and the zip file
            self.clear_temp_folder_and_zip_file(destination, zip_file)

        def _filename_to_module(self, filename: str):
            rootpath = self.get_root_path()
            if filename.startswith(rootpath):
                filename = filename[len(rootpath) :]

            if platform == "macosx":
                prefix = os.sep
            else:
                prefix = os.path.sep

            if filename.startswith(prefix):
                filename = filename[1:]
            module = filename[:-3].replace(prefix, ".")
            return module

else:
    # Android BaseApp
    import hashlib
    import importlib
    import shutil
    import sys

    from kivy.app import App

    class App(App):
        async def main(self) -> None:
            async with trio.open_nursery() as nursery:
                Logger.info("Starting Async Kivy app - Kivy Reloader")
                server = self
                server.nursery = nursery
                await server.async_run("trio")

        def build(self):
            main_py_file_path = os.path.join(os.getcwd(), "main.py")
            if os.path.exists(main_py_file_path):
                self.main_py_hash = self.get_hash_of_file(main_py_file_path)
            else:
                self.main_py_hash = None
            self.kv_files_hashes = {
                file_name: self.get_hash_of_file(file_name)
                for file_name in get_kv_files_paths()
            }
            self.service_files_hashes = {
                file_name: self.get_hash_of_file(file_name)
                for file_name in config.SERVICE_FILES
            }
            self.initialize_server()
            self.recompile_main()

            return self.build_and_reload()

        def build_and_reload(self):
            pass

        def restart_app_on_android(self):
            Logger.info("Restarting the app on smartphone")

            from jnius import autoclass

            Intent = autoclass("android.content.Intent")
            PythonActivity = autoclass("org.kivy.android.PythonActivity")
            System = autoclass("java.lang.System")

            activity = PythonActivity.mActivity
            intent = Intent(activity.getApplicationContext(), PythonActivity)
            intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK)
            activity.startActivity(intent)
            System.exit(0)

        def get_hash_of_file(self, file_name):
            """
            Returns the hash of the file using md5 hash
            """
            with open(file_name, "rb") as f:
                return hashlib.md5(f.read()).hexdigest()

        def _unregister_factory_from_module(self, module):
            to_remove = [x for x in F.classes if F.classes[x]["module"] == module]

            # check class name
            for x in F.classes:
                cls = F.classes[x]["cls"]
                if not cls:
                    continue
                if getattr(cls, "__module__", None) == module:
                    to_remove.append(x)

            for name in set(to_remove):
                del F.classes[name]

        def reload_kv(self, *args):
            """
            Hot reloading kv files on Android
            """
            Logger.info("Reloading kv files")
            main_py_file_path = os.path.join(os.getcwd(), "main.py")

            # reload the service files
            should_restart_app_on_android = False
            for service_name, file_name in zip(
                config.SERVICE_NAMES, config.SERVICE_FILES
            ):
                if (
                    self.get_hash_of_file(file_name)
                    != self.service_files_hashes[file_name]
                ):
                    Logger.info(f"Service {service_name} has been updated")
                    if os.path.exists(f"{file_name}c"):
                        # remove the compiled service file
                        os.remove(f"{file_name}c")

                    # recopiling the service file
                    subprocess.run(f"python -m compileall {file_name}", shell=True)

                    # stop the service
                    Logger.info(f"Stopping service {service_name}")
                    from jnius import autoclass

                    mActivity = autoclass("org.kivy.android.PythonActivity").mActivity
                    context = mActivity.getApplicationContext()
                    SERVICE_NAME = (
                        str(context.getPackageName()) + ".Service" + service_name
                    )
                    service = autoclass(SERVICE_NAME)
                    service.stop(mActivity)
                    should_restart_app_on_android = True

            if should_restart_app_on_android:
                self.restart_app_on_android()
                return

            if os.path.exists(main_py_file_path):
                if self.get_hash_of_file(main_py_file_path) != self.main_py_hash:
                    # `main.py` changed, restarting app
                    self.restart_app_on_android()
                    return

            # Reload only the kv files that changed
            current_kv_files_hashes = {
                file_name: self.get_hash_of_file(file_name)
                for file_name in get_kv_files_paths()
            }

            if current_kv_files_hashes != self.kv_files_hashes:
                kv_files_that_changed = [
                    file_name
                    for file_name in current_kv_files_hashes
                    if current_kv_files_hashes[file_name]
                    != self.kv_files_hashes.get(file_name, None)
                ]

                for file_name in kv_files_that_changed:
                    Builder.unload_file(file_name)
                    Builder.load_file(file_name)

            self.root.clear_widgets()
            root = self.build_and_reload()
            root.do_layout()
            self.root.add_widget(root)

        def unload_python_file(self, filename, module):
            if module in sys.modules:
                full_path = os.path.join(os.getcwd(), filename)
                F.unregister_from_filename(full_path)
                self._unregister_factory_from_module(module)
                importlib.reload(sys.modules[module])

        def unload_python_files_on_android(self):
            for folder in config.WATCHED_FOLDERS_RECURSIVELY:
                for root, _, files in os.walk(folder):
                    for file in files:
                        if file.endswith(".py"):
                            filename = os.path.join(root, file)
                            module = filename.replace("/", ".")[:-3]
                            self.unload_python_file(filename, module)

            for folder in config.WATCHED_FOLDERS:
                for file in os.listdir(folder):
                    if file.endswith(".py"):
                        filename = os.path.join(folder, file)
                        module = filename.replace("/", ".")[:-3]
                        self.unload_python_file(filename, module)

            for file in config.WATCHED_FILES:
                filename = os.path.join(os.getcwd(), file)
                module = filename.replace("/", ".")[:-3]
                self.unload_python_file(filename, module)

        def recompile_main(self):
            if platform == "android":
                files = os.listdir()
                if "main.pyc" in files and "main.py" in files:
                    Logger.info("Deleting main.pyc")
                    os.remove("main.pyc")
                    Logger.info("Compiling main.py")
                    main_py_path = os.path.join(os.getcwd(), "main.py")
                    subprocess.run(f"python -m compileall {main_py_path}", shell=True)

        def initialize_server(self):
            """
            Starts the server
            """
            self.nursery.start_soon(self.start_async_server)

        async def start_async_server(self):
            """
            The android server keeps listening
            expecting the computer to send data to it
            """
            import socket

            try:
                PORT = 8050
                self.s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                self.s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                self.s.connect(("8.8.8.8", 80))

                IP = self.s.getsockname()[0]
                Logger.info(f"Smartphone IP: {IP}")

                await trio.serve_tcp(self.data_receiver, PORT)
            except Exception as e:
                Logger.info(
                    "It was not possible to start the server, check if the phone is connected to the same network as the computer"
                )
                Logger.info(
                    "Another possible cause is that the port is already in use by another app. Check if the port is free and try again"
                )
                Logger.info(e)

        async def data_receiver(self, data_stream):
            """
            When data is received from the computer
            it is saved in a zip file
            and then unpacked
            and the app is reloaded
            """
            Logger.info("Reloader: ************** SERVER **************")
            Logger.info("Reloader: Server started: receiving data from computer...")

            try:
                zip_file_path = os.path.join(os.getcwd(), "app_copy.zip")
                with open(zip_file_path, "wb") as myzip:
                    async for data in data_stream:
                        Logger.info("Reloader: Server: received data")
                        Logger.info(f"Reloader: Data size: {len(data)}")
                        Logger.info("Reloader: Server: connection closed")
                        myzip.write(data)

                Logger.info("Reloader: Finished receiving all files from computer")
                Logger.info("Reloader: Unpacking app")

                # first print the size of the zip file
                zip_file_size = os.path.getsize(zip_file_path)
                Logger.info(f"Reloader: Zip file size: {zip_file_size}")

                shutil.unpack_archive(zip_file_path)

                # Deleting the zip file
                os.remove(zip_file_path)

                # Recompiling main.py
                # Logger.info("Recompiling main.py")
                # self.recompile_main()

                Logger.info("Reloader: App updated, restarting app for refresh")
                Logger.info("Reloader: ************** END SERVER **************")
                self.unload_python_files_on_android()
                self.reload_kv()
            except Exception as e:
                import traceback

                Logger.info(f"Reloader: Server crashed: {e!r}")
                Logger.info(
                    "Full exception:",
                    "".join(traceback.format_exception(*sys.exc_info())),
                )



# The IP address of your phone; You can hot-reload how many phones you want.
# use "127.0.0.1" if you are using an emulator.
PHONE_IPS = ["192.168.1.68"]

# If True, it's going to send the app to smartphone every time you press Ctrl+S
HOT_RELOAD_ON_PHONE = False

# You can stream using "USB" or "WIFI"
# If your USB cable is connected, use "USB", otherwise use "WIFI"
# To use WIFI, you need connect your phone to the same network as your computer
STREAM_USING = "USB"

# The port where the server will be listening
# It can be any port
PORT = 5555

# Add the python files you want to watch here, i.e., if they change, the app will be reloaded
WATCHED_FILES = ["main.py"]

# Add the folders you want to watch recursively here, i.e., if **ANY FILE** inside them changes, the app will be reloaded
WATCHED_FOLDERS_RECURSIVELY = []

# Add the folders where you have your .kv files here
# If you don't put them here, the .kv files won't be reloaded
# this is recursive, i.e., it will watch all folders inside the folders you put here
WATCHED_KV_FOLDERS_RECURSIVELY = []

# Watched folders but not recursively, i.e., **ANY FILE** inside the first level of the folder
WATCHED_FOLDERS = []

# Add the folders where you have your .kv files here
# If you don't put them here, the .kv files won't be reloaded
# this is not recursive, i.e., it will only watch the first level of the folders you put here
WATCHED_KV_FOLDERS = []

# If you want to logcat services, put here the name of each service
# For example, ["MyService", "MyOtherService"]
SERVICE_NAMES = []

# If you want to hotreload your services, put the name of the service files here
# For example, ["service.py", "service2.py"]
SERVICE_FILES = []

# If you want to exclude some files or folders, from being copied to the phone, add them here
FOLDERS_AND_FILES_TO_EXCLUDE_FROM_PHONE = [
    "*.pyc",
    "__pycache__",
    ".buildozer",
    ".venv",
    ".vscode",
    ".git",
    ".pytest_cache",
    ".DS_Store",
    ".env",
    ".dmypy.json",
    ".mypy_cache/",
    ".dmypy.json",
    "dmypy.json",
    "env/",
    "ENV/",
    "env.bak/",
    "venv/",
    "venv.bak/",
    "bin",
    "buildozer.spec",
    "poetry.lock",
    "pyproject.toml",
    "temp",
    "tests",
    "app_copy.zip",
    "send_app_to_phone.py",
    ".gitignore",
    "README.md",
]

# The X position of the window generated by scrcpy
WINDOW_X = "1200"

# The Y position of the window generated by scrcpy
WINDOW_Y = "100"

# The width of the window generated by scrcpy
WINDOW_WIDTH = "280"

# The title of the window generated by scrcpy
WINDOW_TITLE = "Kivy Reloader"

# If you want to show the touches on the screen, set this to True
SHOW_TOUCHES = False

# If you want the screen to be always on, set this to True
STAY_AWAKE = False

# If you want to turn the screen off, set this to True
TURN_SCREEN_OFF = False

# If you want the window to be always on top, set this to True
ALWAYS_ON_TOP = True

assert (
    len(WATCHED_FILES) + len(WATCHED_FOLDERS) + len(WATCHED_FOLDERS_RECURSIVELY) > 0
), "No files or folders to watch"



import os

from kivy.lang import Builder

from .config import config

base_dir = os.getcwd()


def load_kv_path(path):
    """
    Loads a kv file from a path
    """
    kv_path = os.path.join(base_dir, path)
    if kv_path in Builder.files:
        Builder.unload_file(kv_path)

    if kv_path not in Builder.files:
        Builder.load_file(kv_path)


def get_auto_reloader_paths():
    """
    Returns a list of paths to watch for changes,
    based on the constants.py file
    """
    return (
        [(os.path.join(base_dir, x), {"recursive": True}) for x in config.WATCHED_FILES]
        + [
            (os.path.join(base_dir, x), {"recursive": True})
            for x in config.WATCHED_FOLDERS_RECURSIVELY
        ]
        + [
            (os.path.join(base_dir, x), {"recursive": False})
            for x in config.WATCHED_FOLDERS
        ]
    )


def find_kv_files_in_folder(folder):
    kv_files = []
    for root, _, files in os.walk(os.path.join(base_dir, folder)):
        for file in files:
            if file.endswith(".kv"):
                kv_files.append(os.path.join(root, file))
    return kv_files


def get_kv_files_paths():
    """
    Given the folders on WATCHED_KV_FOLDERS and WATCHED_KV_FOLDERS_RECURSIVELY,
    returns a list of all the kv files paths
    """
    KV_FILES = []

    for folder in config.WATCHED_KV_FOLDERS:
        for kv_file in os.listdir(folder):
            if kv_file.endswith(".kv"):
                KV_FILES.append(os.path.join(base_dir, f"{folder}/{kv_file}"))

    for folder in config.WATCHED_KV_FOLDERS_RECURSIVELY:
        for kv_file in find_kv_files_in_folder(folder):
            KV_FILES.append(kv_file)

    # Removing duplicates
    KV_FILES = list(set(KV_FILES))

    return KV_FILES
